/*
 * AProgramThatPrintsOutAnotherProgram.c
 *
 * This program prints out Algorithm 1 in Assignment 4, but with the
 * necessary modifications to print all data types and operator pairs.
 * This is done by printing Algorithm 1 to a terminal, like PuTTY.
 * Getting the timing results is a two step process.  First compile
 * this program and save its output from the PIC to a file on your computer.
 * Then compile the output into a program and run it on the PIC.  The results
 * will then go into the table for 1.1 (a).
 *
 * For detailed information on how to do this with PuTTY, see README.pdf.
 *
 * HISTORY:
 * 02/13/2011  Created by Nelson Rosa
 *
 */

#include <plib.h>
#include "NU32v2.h"

#define NBUF 1024
#define NTYPES 6
#define NOPS 4
#define REPEAT 20

// the C data types we want to time
char *types[] = {
  "char",
  "short",
  "int",
  "long long",
  "float",
  "double" 
};

// the mathematical operators we want to time
char *ops[] = {
  "+",
  "-",
  "*",
  "/"
};

int main() {
  int i,j,k;
  char buffer[NBUF];  
  
  // open the serial port for use with Putty
  initSerialNU32v2();
  
  // Print out Algorithm 1, but with the necessary changes

  // header info
  sprintf(buffer, "#include <plib.h>\r\n");
  WriteString(UART3, buffer);
  
  sprintf(buffer, "#include \"NU32v2.h\"\r\n");
  WriteString(UART3, buffer);
  
  sprintf(buffer, "\r\n\r\n");
  WriteString(UART3, buffer);
  
  // timing functions, the function name follows the sequence:
  // timeCodeXY, where X is the index of the data type and Y
  // is the index of the operator in the types and ops arrays
  for(i = 0; i < NTYPES; i++) {
   for(j = 0; j < NOPS; j++) {
     sprintf(buffer, "%s timeCode%d%d() {\r\n", types[i], i, j);
     WriteString(UART3, buffer);
     
     sprintf(buffer, "  char buffer[1024];\r\n");
     WriteString(UART3, buffer);
     
     sprintf(buffer, "  %s ans, a, b;\r\n", types[i]);
     WriteString(UART3, buffer);
     
     sprintf(buffer, "  unsigned int e;\r\n");
     WriteString(UART3, buffer);
     
     sprintf(buffer, "  a = 17;\r\n");
     WriteString(UART3, buffer);
     
     sprintf(buffer, "  b = 12;\r\n");
     WriteString(UART3, buffer);
     
     sprintf(buffer, "  OpenTimer2(T2_ON|T2_SOURCE_INT|T2_PS_1_1, 0xFFFF);\r\n");
     WriteString(UART3, buffer);
     
	 // perform each operation REPEAT times
     for(k = 0; k < REPEAT; k++) {
       sprintf(buffer, "  ans = a %s b;\r\n", ops[j]);
       WriteString(UART3, buffer);
     }
     
     sprintf(buffer, "  e = ReadTimer2();\r\n");
     WriteString(UART3, buffer);
     

     sprintf(buffer, "  sprintf(buffer, \r\n");
     WriteString(UART3, buffer);

     // the following format string shows how to print out special
	 // characters.  For example, to print a quotation mark (") use \"
	 // and to print a percent sign (%) use %%.
     sprintf(buffer, 
       "    \"%s (%s): %%f (total: %%d)\\r\\n\", e/%d.0, e);\r\n",
       types[i], ops[j], REPEAT);
     WriteString(UART3, buffer);
     
     sprintf(buffer, "  WriteString(UART3, buffer);\r\n");
     WriteString(UART3, buffer);
     
     sprintf(buffer, "  return ans;\r\n");
     WriteString(UART3, buffer);
     
     sprintf(buffer, "}\r\n");
     WriteString(UART3, buffer);
     
     sprintf(buffer, "\r\n");
     WriteString(UART3, buffer);
   }
  }
  
  // print a main function that calls every function we've created.
  // there is no need to declare prototypes if main() comes at the
  // end of a program after defining all functions.
  sprintf(buffer, "int main() {\r\n");
  WriteString(UART3, buffer);

  sprintf(buffer, "  initSerialNU32v2();\r\n");
  WriteString(UART3, buffer);
  
  for(i = 0; i < NTYPES; i++) {
    for(j = 0; j < NOPS; j++) {
      sprintf(buffer, "  timeCode%d%d();\r\n", i, j);
      WriteString(UART3, buffer);
    }
  }
  
  sprintf(buffer, "  return 0;\r\n");
  WriteString(UART3, buffer);
  
  sprintf(buffer, "}\r\n");
  WriteString(UART3, buffer);
  
  sprintf(buffer, "\r\n");
  WriteString(UART3, buffer);  
  
 return 0; 
}
